/*
Orion's Standard Library
Orion Sky Lawlor, 9/23/1999

Provides routines for creating, reading in,
adding, multiplying, and inverting 3d
rotation/translation/scale/skew matrices.
*/
#ifndef __OSL_MATRIX_3D_H
#define __OSL_MATRIX_3D_H

#ifndef __OSL_H
#  include "osl/osl.h"
#endif
#ifndef __OSL_MATRIX_T_H
#include "osl/matrixT.h"
#endif
#ifndef __OSL_VECTOR_3D_H
#include "osl/vector3d.h"
#endif

namespace osl {

typedef float Matrix3d_real;
class Matrix3d : public MatrixT<Matrix3d_real,4,4> {
public:
	typedef Matrix3d_real flt;
	Matrix3d() {}//Builds 4x4 identity Matrix)
	
	Matrix3d(flt s) {identity(s);}	
	
	void identity(flt s=1.0);


	//Create Matrix with given column vectors
	Matrix3d(const Vector3d &x,const Vector3d &y,
		const Vector3d &z,const Vector3d &origin) {
		setX(x);setY(y);setZ(z);setO(origin);
		data[3][0]=data[3][1]=data[3][2]=(flt)0.0;data[3][3]=(flt)1.0;
	}
	
	void setCol(const Vector3d &a,int colNo) 
		{data[0][colNo]=(flt)a.x;data[1][colNo]=(flt)a.y;data[2][colNo]=(flt)a.z;}
	Vector3d getCol(int colNo) const
		{return Vector3d(data[0][colNo],data[1][colNo],data[2][colNo]);}		
	void setX(const Vector3d &a) {setCol(a,0);}
	void setY(const Vector3d &a) {setCol(a,1);}
	void setZ(const Vector3d &a) {setCol(a,2);}
	void setO(const Vector3d &a) {setCol(a,3);}	
	
	//Make this Matrix a rotation Matrix, right-handed about +x axis
	// (*must* have just called Matrix3d() constructor)
	void rotateX(real radians);
	void rotateY(real radians);
	void rotateZ(real radians);
	
	//Shift this Matrix' origin by the given distance
	void translate(const Vector3d &off)
	  {data[0][3]+=(flt)off.x;data[1][3]+=(flt)off.y;data[2][3]+=(flt)off.z;}
	//Scale this Matrix up by the specified factors
	void scale(const Vector3d &fac);
	
	//Apply this Matrix to the given vector 
	Vector3d apply(const Vector3d &in) const;
	//Apply this Matrix to the given direction vector (no offset added)
	Vector3d applyDirection(const Vector3d &in) const;
	
	//Inline versions of the above calls
	Vector3d applyInline(const Vector3d &in) const {
	   Vector3d dest;
	   dest.x=data[0][0]*in.x+data[0][1]*in.y+data[0][2]*in.z+data[0][3];
	   dest.y=data[1][0]*in.x+data[1][1]*in.y+data[1][2]*in.z+data[1][3];
	   dest.z=data[2][0]*in.x+data[2][1]*in.y+data[2][2]*in.z+data[2][3];
	   return dest;
	}
	Vector3d applyDirectionInline(const Vector3d &in) const {
	   Vector3d dest;
	   dest.x=data[0][0]*in.x+data[0][1]*in.y+data[0][2]*in.z;
	   dest.y=data[1][0]*in.x+data[1][1]*in.y+data[1][2]*in.z;
	   dest.z=data[2][0]*in.x+data[2][1]*in.y+data[2][2]*in.z;
	   return dest;
	}
};

}; //end namespace osl
#endif
